
// from redux code
function compose(...funcs) {
  if (funcs.length === 0) {
      return arg => arg
  }

  if (funcs.length === 1) {
      return funcs[0]
  }

  return funcs.reduce((a, b) => (...args) => a(b(...args)))
}

function createPlugin() {
  const hooks = {};

  const apply = (type, func) => {
    let listeners = hooks[type] || []
    listeners = listeners.filter(x => x !== func).concat(func)
    hooks[type] = listeners
  }

  const invoke = (type, ...args) => {
    const listeners = hooks[type] || []
    return compose(...listeners)(...args)
  }

  return { apply, invoke }
}

const plugins = createPlugin()
export default plugins
