import { createRouter, createWebHashHistory } from 'vue-router';
import NProgress from 'nprogress'
import NoteLayout from './layouts/NoteLayout.vue'
import store from './store'

const routes = [
  {
    path: '/',
    redirect: '/index',
    component: NoteLayout,
    children: [
      { path: 'index', name: 'index', component: () => import('./pages/index.vue') }
    ]
  },
  {
    path: '/login',
    name: 'login',
    component: () => import('./pages/login/index.vue')
  }
]

const router = createRouter({
  // 4. 内部提供了 history 模式的实现。为了简单起见，我们在这里使用 hash 模式。
  history: createWebHashHistory(),
  routes, // `routes: routes` 的缩写
})

const isEmpty = x => !x || !x.length || !Object.keys(x).length

router.beforeEach((to, from, next) => {
  NProgress.start();
  const { user, getUserInfo } = store.getState('user')

  if (to.name === 'login') {
    NProgress.done();
    return next()
  }

  if (isEmpty(user)) {    
    getUserInfo(user.token).then(() => {
      NProgress.done();
      next()
    }).catch(() => {
      NProgress.done();
      next({ name: 'login' })
    })
    return
  } else {
    NProgress.done();
    next()
  }
  
})

export default router;
