import { reactive, ref, computed } from 'vue'
import { message } from 'ant-design-vue';
import * as gitee from '@/service/gitee'
import plugins from '@/plugins'

const getParentPath = path => {
  const paths = path.split('/')
  paths.pop()
  return paths.join('/')       
}

const decodeData = raw => plugins.invoke('readNote', raw)
const encodeData = raw => plugins.invoke('writeNote', raw)

export default function useNote() {
  const fileArr = reactive([])
  // const fileTree = reactive([])
  const note = ref()

  // 获取目录下所有问题件
  const getFiles = (user, path, repo = 'my-note') => {
    return gitee.getFiles(user, path, repo).then(res => {
      const files = (res || []).filter (x => x.path.indexOf('.gitignore') === -1)

      if (!path) {
        fileArr.splice(0)
        fileArr.push(...files)
      } else {
        const dir = fileArr.filter(x => x.type === 'dir' && x.path === path)[0]
        dir.children = files
        fileArr.push(...files)
      }
    })
  }

  const fileTree = computed(() => {
    return fileArr.filter(x => x.path === x.name)
  })

  const getNote = (user, path = '', repo = 'my-note') => {
    return gitee.getFile(user, path, repo).then(res => {
      const content = res.content ? decodeData(res.content) : undefined

      note.value = {
        ...res,
        content
      }
    })
  }

  const newNote = (user, path, repo = 'my-note') => {
    const content = encodeData('又是一篇崭新的日记...')
    gitee.newFile(user, content, path, repo).then(() => {
      getFiles(user, getParentPath(path), repo)
    })
  }

  const updateNote = (content, user, repo = 'my-note') => {
    const path = note.value.path
    const fileInfo = { path, sha: note.value.sha, content: encodeData(content) }

    return gitee.updateFile(user, fileInfo, repo).then(() => {
      message.success('保存成功');
      // 获取最新的数据, 目的是更新 sha 值
      gitee.getFile(user, path, repo).then(res => {
        const content = res.content ? decodeData(res.content) : undefined
  
        note.value = {
          ...res,
          content
        }
      })
    })
  }

  const newFolder = (user, path, repo = 'my-note') => {
    gitee.newFolder(user, path, repo).then(() => {
      getFiles(user, getParentPath(path), repo)
    })
  }

  const deleteNote = (user, path, repo = 'my-note') => {
    let pos = 0
    let file = undefined
    for (; pos < fileArr.length; pos ++) {
      if (fileArr[pos].path === path) {
        file = fileArr[pos]
        break;
      }
    }

    if (!file) return;

    gitee.deleteFile(user, { path, sha: file.sha }, repo).then(() => {
      fileArr.splice(pos, 1)
    })
  }
  
  return {
    fileArr,
    fileTree,
    note,
    getFiles,
    newFolder,
    getNote,
    newNote,
    updateNote,
    deleteNote,
  }
}
