import { reactive } from 'vue'
import * as gitee from '@/service/gitee'
import { getToken } from '@/utils/token'

export default function useUser() {
  const user = reactive({})

  const getUserInfo = () => {
    return gitee.getCurrentUser().then(res => {
      Object.assign(user, res || {})
    })
  }

  const signIn = (data) => {
    return gitee.signIn(data).then(getUserInfo)
  }
  
  const signOut = () => {
    // xxx
  }


  // 快过期的需要刷新 token
  const refreshToken = () => {
    const token = getToken()
    const now = (new Date()).valueOf() / 1000
    const expire = token.expires_in + token.created_at
    const diff = 1 * 60 * 60 // 小时

    if (now + diff > expire) {
      return gitee.refreshToken(token)
    }
  }
  
  const t = setTimeout(() => {
    const token = getToken()
    clearTimeout(t)

    if (token) {
      refreshToken()
    }
  }, 0)

  return {
    user,
    signIn,
    signOut,
    getUserInfo
  }
}
